@@ -15,7 +15,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/LINUX/osi_module.c,v 1.74.2.7 2007/07/09 20:00:35 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/LINUX/osi_module.c,v 1.74.2.8 2007/08/22 02:17:52 shadow Exp $");
 
 #include <linux/module.h> /* early to avoid printf->printk mapping */
 #include "afs/sysincludes.h"
@@ -94,11 +94,20 @@ init_module(void)
 	return err;
 #endif
     err = afs_init_inodecache();
-    if (err)
+    if (err) {
+#ifndef LINUX_KEYRING_SUPPORT
+	osi_syscall_clean();
+#endif
 	return err;
+    }
     err = register_filesystem(&afs_fs_type);
-    if (err)
-      return err;
+    if (err) {
+	afs_destroy_inodecache();
+#ifndef LINUX_KEYRING_SUPPORT
+	osi_syscall_clean();
+#endif
+	return err;
+    }
 
     osi_sysctl_init();
 #ifdef LINUX_KEYRING_SUPPORT
@@ -120,9 +129,13 @@ void
 cleanup_module(void)
 #endif
 {
+#ifdef LINUX_KEYRING_SUPPORT
     osi_keyring_shutdown();
+#endif
     osi_sysctl_clean();
+#ifndef LINUX_KEYRING_SUPPORT
     osi_syscall_clean();
+#endif
     unregister_filesystem(&afs_fs_type);
 
     afs_destroy_inodecache();
