@@ -21,7 +21,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_write.c,v 1.35 2003/08/29 22:00:04 rees Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_write.c,v 1.50 2006/04/04 01:35:15 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -105,8 +105,13 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
     volatile
 #endif
     afs_int32 error;
+#ifdef AFS_DARWIN80_ENV
+    uio_t tuiop = NULL;
+#else
     struct uio tuio;
+    struct uio *tuiop = &tuio;
     struct iovec *tvec;		/* again, should have define */
+#endif
     register afs_int32 code;
     struct vrequest treq;
 
@@ -118,8 +123,8 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
     if ((code = afs_InitReq(&treq, acred)))
 	return code;
     /* otherwise we read */
-    totalLength = auio->afsio_resid;
-    filePos = auio->afsio_offset;
+    totalLength = AFS_UIO_RESID(auio);
+    filePos = AFS_UIO_OFFSET(auio);
     error = 0;
     transferLength = 0;
     afs_Trace4(afs_iclSetp, CM_TRACE_WRITE, ICL_TYPE_POINTER, avc,
@@ -141,8 +146,8 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
 	 */
 	osi_Assert(filePos <= avc->m.Length);
 	diff = avc->m.Length - filePos;
-	auio->afsio_resid = MIN(totalLength, diff);
-	totalLength = auio->afsio_resid;
+	AFS_UIO_SETRESID(auio, MIN(totalLength, diff));
+	totalLength = AFS_UIO_RESID(auio);
     }
 #else
     if (aio & IO_APPEND) {
@@ -150,7 +155,8 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
 #if	defined(AFS_SUN56_ENV)
 	auio->uio_loffset = 0;
 #endif
-	filePos = auio->afsio_offset = avc->m.Length;
+	filePos = avc->m.Length;
+	AFS_UIO_SETOFFSET(auio, filePos);
     }
 #endif
     /*
@@ -159,16 +165,12 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
      */
     avc->m.Date = startDate;
 
-#if	defined(AFS_HPUX_ENV) || defined(AFS_GFS_ENV)
+#if	defined(AFS_HPUX_ENV)
 #if	defined(AFS_HPUX101_ENV)
     if ((totalLength + filePos) >> 9 >
 	(p_rlimit(u.u_procp))[RLIMIT_FSIZE].rlim_cur) {
 #else
-#ifdef	AFS_HPUX_ENV
     if ((totalLength + filePos) >> 9 > u.u_rlimit[RLIMIT_FSIZE].rlim_cur) {
-#else
-    if (totalLength + filePos > u.u_rlimit[RLIMIT_FSIZE].rlim_cur) {
-#endif
 #endif
 	if (!noLock)
 	    ReleaseWriteLock(&avc->lock);
@@ -181,27 +183,30 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
      * high-level write op.
      */
     if (avc->execsOrWriters <= 0) {
-	printf("WARNING: afs_ufswr vp=%x, exOrW=%d\n", avc,
+	printf("WARNING: afs_ufswr vp=%lx, exOrW=%d\n", (unsigned long)avc,
 	       avc->execsOrWriters);
     }
 #else
     afs_FakeOpen(avc);
 #endif
     avc->states |= CDirty;
+#ifndef AFS_DARWIN80_ENV
     tvec = (struct iovec *)osi_AllocSmallSpace(sizeof(struct iovec));
+#endif
     while (totalLength > 0) {
-	/* Read the cached info. If we call GetDCache while the cache
-	 * truncate daemon is running we risk overflowing the disk cache.
-	 * Instead we check for an existing cache slot. If we cannot
-	 * find an existing slot we wait for the cache to drain
-	 * before calling GetDCache.
+	/* 
+	 *  The following line is necessary because afs_GetDCache with
+	 *  flag == 4 expects the length field to be filled. It decides
+	 *  from this whether it's necessary to fetch data into the chunk
+	 *  before writing or not (when the whole chunk is overwritten!).
 	 */
+	len = totalLength;	/* write this amount by default */
 	if (noLock) {
 	    tdc = afs_FindDCache(avc, filePos);
 	    if (tdc)
 		ObtainWriteLock(&tdc->lock, 653);
 	} else if (afs_blocksUsed >
-		   (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100) {
+		   PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
 	    tdc = afs_FindDCache(avc, filePos);
 	    if (tdc) {
 		ObtainWriteLock(&tdc->lock, 654);
@@ -215,10 +220,10 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
 	    if (!tdc) {
 		afs_MaybeWakeupTruncateDaemon();
 		while (afs_blocksUsed >
-		       (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100) {
+		       PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
 		    ReleaseWriteLock(&avc->lock);
 		    if (afs_blocksUsed - afs_blocksDiscarded >
-			(CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100) {
+			PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
 			afs_WaitForCacheDrain = 1;
 			afs_osi_Sleep(&afs_WaitForCacheDrain);
 		    }
@@ -257,13 +262,21 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
 	     * as will fit */
 	    len = max - offset;
 	}
+
+#ifdef  AFS_DARWIN80_ENV
+        if (tuiop)
+	    uio_free(tuiop);
+	trimlen = len;
+	tuiop = afsio_darwin_partialcopy(auio, trimlen);
+#else
 	/* mung uio structure to be right for this transfer */
 	afsio_copy(auio, &tuio, tvec);
 	trimlen = len;
 	afsio_trim(&tuio, trimlen);
-	tuio.afsio_offset = offset;
+#endif
+	AFS_UIO_SETOFFSET(tuiop, offset);
 
-	code = afs_MemWriteUIO(tdc->f.inode, &tuio);
+	code = afs_MemWriteUIO(tdc->f.inode, tuiop);
 	if (code) {
 	    void *mep;		/* XXX in prototype world is struct memCacheEntry * */
 	    error = code;
@@ -278,7 +291,7 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
 	    break;
 	}
 	/* otherwise we've written some, fixup length, etc and continue with next seg */
-	len = len - tuio.afsio_resid;	/* compute amount really transferred */
+	len = len - AFS_UIO_RESID(tuiop);	/* compute amount really transferred */
 	tlen = len;
 	afsio_skip(auio, tlen);	/* advance auio over data written */
 	/* compute new file size */
@@ -326,11 +339,10 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
 	avc->vc_error = error;
     if (!noLock)
 	ReleaseWriteLock(&avc->lock);
+#ifdef AFS_DARWIN80_ENV
+    uio_free(tuiop);
+#else
     osi_FreeSmallSpace(tvec);
-#ifdef AFS_DEC_ENV
-    /* next, on GFS systems, we update g_size so that lseek's relative to EOF will
-     * work.  GFS is truly a poorly-designed interface!  */
-    afs_gfshack((struct gnode *)avc);
 #endif
     error = afs_CheckCode(error, &treq, 6);
     return error;
@@ -355,8 +367,13 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
     volatile
 #endif
     afs_int32 error;
+#ifdef AFS_DARWIN80_ENV
+    uio_t tuiop = NULL;
+#else
     struct uio tuio;
+    struct uio *tuiop = &tuio;
     struct iovec *tvec;		/* again, should have define */
+#endif
     struct osi_file *tfile;
     register afs_int32 code;
     struct vrequest treq;
@@ -369,8 +386,8 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
     if ((code = afs_InitReq(&treq, acred)))
 	return code;
     /* otherwise we read */
-    totalLength = auio->afsio_resid;
-    filePos = auio->afsio_offset;
+    totalLength = AFS_UIO_RESID(auio);
+    filePos = AFS_UIO_OFFSET(auio);
     error = 0;
     transferLength = 0;
     afs_Trace4(afs_iclSetp, CM_TRACE_WRITE, ICL_TYPE_POINTER, avc,
@@ -392,8 +409,8 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 	 */
 	osi_Assert(filePos <= avc->m.Length);
 	diff = avc->m.Length - filePos;
-	auio->afsio_resid = MIN(totalLength, diff);
-	totalLength = auio->afsio_resid;
+	AFS_UIO_SETRESID(auio, MIN(totalLength, diff));
+	totalLength = AFS_UIO_RESID(auio);
     }
 #else
     if (aio & IO_APPEND) {
@@ -401,7 +418,8 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 #if     defined(AFS_SUN56_ENV)
 	auio->uio_loffset = 0;
 #endif
-	filePos = auio->afsio_offset = avc->m.Length;
+	filePos = avc->m.Length;
+	AFS_UIO_SETOFFSET(auio, avc->m.Length);
     }
 #endif
     /*
@@ -410,16 +428,12 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
      */
     avc->m.Date = startDate;
 
-#if	defined(AFS_HPUX_ENV) || defined(AFS_GFS_ENV)
+#if	defined(AFS_HPUX_ENV)
 #if 	defined(AFS_HPUX101_ENV)
     if ((totalLength + filePos) >> 9 >
 	p_rlimit(u.u_procp)[RLIMIT_FSIZE].rlim_cur) {
 #else
-#ifdef	AFS_HPUX_ENV
     if ((totalLength + filePos) >> 9 > u.u_rlimit[RLIMIT_FSIZE].rlim_cur) {
-#else
-    if (totalLength + filePos > u.u_rlimit[RLIMIT_FSIZE].rlim_cur) {
-#endif
 #endif
 	if (!noLock)
 	    ReleaseWriteLock(&avc->lock);
@@ -432,14 +446,16 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
      * high-level write op.
      */
     if (avc->execsOrWriters <= 0) {
-	printf("WARNING: afs_ufswr vcp=%x, exOrW=%d\n", avc,
+	printf("WARNING: afs_ufswr vcp=%lx, exOrW=%d\n", (unsigned long)avc,
 	       avc->execsOrWriters);
     }
 #else
     afs_FakeOpen(avc);
 #endif
     avc->states |= CDirty;
+#ifndef AFS_DARWIN80_ENV
     tvec = (struct iovec *)osi_AllocSmallSpace(sizeof(struct iovec));
+#endif
     while (totalLength > 0) {
 	/* 
 	 *  The following line is necessary because afs_GetDCache with
@@ -454,7 +470,7 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 	    if (tdc)
 		ObtainWriteLock(&tdc->lock, 657);
 	} else if (afs_blocksUsed >
-		   (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100) {
+		   PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
 	    tdc = afs_FindDCache(avc, filePos);
 	    if (tdc) {
 		ObtainWriteLock(&tdc->lock, 658);
@@ -468,10 +484,10 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 	    if (!tdc) {
 		afs_MaybeWakeupTruncateDaemon();
 		while (afs_blocksUsed >
-		       (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100) {
+		       PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
 		    ReleaseWriteLock(&avc->lock);
 		    if (afs_blocksUsed - afs_blocksDiscarded >
-			(CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100) {
+			PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
 			afs_WaitForCacheDrain = 1;
 			afs_osi_Sleep(&afs_WaitForCacheDrain);
 		    }
@@ -511,11 +527,20 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 	     * as will fit */
 	    len = max - offset;
 	}
+
+#ifdef  AFS_DARWIN80_ENV
+	if (tuiop)
+	    uio_free(tuiop);
+	trimlen = len;
+	tuiop = afsio_darwin_partialcopy(auio, trimlen);
+#else
 	/* mung uio structure to be right for this transfer */
 	afsio_copy(auio, &tuio, tvec);
 	trimlen = len;
 	afsio_trim(&tuio, trimlen);
-	tuio.afsio_offset = offset;
+#endif
+	AFS_UIO_SETOFFSET(tuiop, offset);
+
 #if defined(AFS_AIX41_ENV)
 	AFS_GUNLOCK();
 	code =
@@ -530,9 +555,19 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 		      &tuio, NULL, NULL, -1);
 #elif defined(AFS_SUN5_ENV)
 	AFS_GUNLOCK();
+#ifdef AFS_SUN510_ENV
+	{
+	    caller_context_t ct;
+
+	    VOP_RWLOCK(tfile->vnode, 1, &ct);
+	    code = VOP_WRITE(tfile->vnode, &tuio, 0, afs_osi_credp, &ct);
+	    VOP_RWUNLOCK(tfile->vnode, 1, &ct);
+	}
+#else
 	VOP_RWLOCK(tfile->vnode, 1);
 	code = VOP_WRITE(tfile->vnode, &tuio, 0, afs_osi_credp);
 	VOP_RWUNLOCK(tfile->vnode, 1);
+#endif
 	AFS_GLOCK();
 	if (code == ENOSPC)
 	    afs_warnuser
@@ -563,7 +598,11 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 	}
 #elif defined(AFS_LINUX20_ENV)
 	AFS_GUNLOCK();
-	code = osi_file_uio_rdwr(tfile, &tuio, UIO_WRITE);
+	code = osi_rdwr(tfile, &tuio, UIO_WRITE);
+	AFS_GLOCK();
+#elif defined(AFS_DARWIN80_ENV)
+	AFS_GUNLOCK();
+	code = VNOP_WRITE(tfile->vnode, tuiop, 0, afs_osi_ctxtp);
 	AFS_GLOCK();
 #elif defined(AFS_DARWIN_ENV)
 	AFS_GUNLOCK();
@@ -602,7 +641,7 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 	    break;
 	}
 	/* otherwise we've written some, fixup length, etc and continue with next seg */
-	len = len - tuio.afsio_resid;	/* compute amount really transferred */
+	len = len - AFS_UIO_RESID(tuiop);	/* compute amount really transferred */
 	tlen = len;
 	afsio_skip(auio, tlen);	/* advance auio over data written */
 	/* compute new file size */
@@ -653,11 +692,10 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 	avc->vc_error = error;
     if (!noLock)
 	ReleaseWriteLock(&avc->lock);
+#ifdef AFS_DARWIN80_ENV
+    uio_free(tuiop);
+#else
     osi_FreeSmallSpace(tvec);
-#ifdef AFS_DEC_ENV
-    /* next, on GFS systems, we update g_size so that lseek's relative to EOF will
-     * work.  GFS is truly a poorly-designed interface!  */
-    afs_gfshack((struct gnode *)avc);
 #endif
 #ifndef	AFS_VM_RDWR_ENV
     /*
@@ -704,9 +742,7 @@ afs_DoPartialWrite(register struct vcache *avc, struct vrequest *areq)
     return code;
 }
 
-
-
-#if !defined (AFS_AIX_ENV) && !defined (AFS_HPUX_ENV) && !defined (AFS_SUN5_ENV) && !defined(AFS_SGI_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN_ENV) && !defined(AFS_XBSD_ENV)
+#ifdef AFS_OSF_ENV
 #ifdef AFS_DUX50_ENV
 #define vno_close(X) vn_close((X), 0, NOCRED)
 #elif defined(AFS_DUX40_ENV)
@@ -760,17 +796,9 @@ afs_closex(register struct file *afd)
 	    afd->f_flag &= ~(FSHLOCK | FEXLOCK);
 	    code = vno_close(afd);
 	    if (flags)
-#if defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV) || defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV)
 		HandleFlock(tvc, LOCK_UN, &treq, u.u_procp->p_pid,
 			    1 /*onlymine */ );
-#else
-		HandleFlock(tvc, LOCK_UN, &treq, 0, 1 /*onlymine */ );
-#endif
-#ifdef	AFS_DEC_ENV
-	    grele((struct gnode *)tvc);
-#else
 	    AFS_RELE(AFSTOV(tvc));
-#endif
 	    closeDone = 1;
 	}
     }
@@ -803,18 +831,14 @@ afs_close(OSI_VC_ARG(avc), aflags, lastclose,
      struct flid *flp;
 #endif
 #endif
-#elif	defined(AFS_SUN_ENV) || defined(AFS_SUN5_ENV)
-#ifdef	AFS_SUN5_ENV
+#elif defined(AFS_SUN5_ENV)
 afs_close(OSI_VC_ARG(avc), aflags, count, offset, acred)
      offset_t offset;
-#else
-afs_close(OSI_VC_ARG(avc), aflags, count, acred)
-#endif
      int count;
 #else
 afs_close(OSI_VC_ARG(avc), aflags, acred)
 #endif
-OSI_VC_DECL(avc);
+     OSI_VC_DECL(avc);
      afs_int32 aflags;
      struct AFS_UCRED *acred;
 {
@@ -825,9 +849,9 @@ OSI_VC_DECL(avc);
     struct flid flid;
 #endif
     struct afs_fakestat_state fakestat;
-    OSI_VC_CONVERT(avc)
+    OSI_VC_CONVERT(avc);
 
-	AFS_STATCNT(afs_close);
+    AFS_STATCNT(afs_close);
     afs_Trace2(afs_iclSetp, CM_TRACE_CLOSE, ICL_TYPE_POINTER, avc,
 	       ICL_TYPE_INT32, aflags);
     code = afs_InitReq(&treq, acred);
@@ -849,15 +873,6 @@ OSI_VC_DECL(avc);
 	afs_PutFakeStat(&fakestat);
 	return 0;
     }
-#elif	defined(AFS_SUN_ENV) || defined(AFS_SGI_ENV)
-    if (count > 1) {
-	/* The vfs layer may call this repeatedly with higher "count"; only on the last close (i.e. count = 1) we should actually proceed with the close. */
-	afs_PutFakeStat(&fakestat);
-	return 0;
-    }
-#endif
-#ifndef	AFS_SUN5_ENV
-#if defined(AFS_SGI_ENV)
     /* unlock any locks for pid - could be wrong for child .. */
     AFS_RWLOCK((vnode_t *) avc, VRWLOCK_WRITE);
 #ifdef AFS_SGI65_ENV
@@ -874,16 +889,21 @@ OSI_VC_DECL(avc);
 #endif /* AFS_SGI65_ENV */
     /* afs_chkpgoob will drop and re-acquire the global lock. */
     afs_chkpgoob(&avc->v, btoc(avc->m.Length));
+#elif	defined(AFS_SUN5_ENV)
+    if (count > 1) {
+	/* The vfs layer may call this repeatedly with higher "count"; only on the last close (i.e. count = 1) we should actually proceed with the close. */
+	afs_PutFakeStat(&fakestat);
+	return 0;
+    }
 #else /* AFS_SGI_ENV */
     if (avc->flockCount) {	/* Release Lock */
-#if	defined(AFS_OSF_ENV) || defined(AFS_SUN_ENV)
+#if	defined(AFS_OSF_ENV) 
 	HandleFlock(avc, LOCK_UN, &treq, u.u_procp->p_pid, 1 /*onlymine */ );
 #else
 	HandleFlock(avc, LOCK_UN, &treq, 0, 1 /*onlymine */ );
 #endif
     }
 #endif /* AFS_SGI_ENV */
-#endif /* AFS_SUN5_ENV */
     if (aflags & (FWRITE | FTRUNC)) {
 	if (afs_BBusy()) {
 	    /* do it yourself if daemons are all busy */
@@ -983,34 +1003,24 @@ OSI_VC_DECL(avc);
 
 int
 #ifdef	AFS_OSF_ENV
-afs_fsync(avc, fflags, acred, waitfor)
-     int fflags;
-     int waitfor;
+afs_fsync(OSI_VC_DECL(avc), int fflags, struct AFS_UCRED *acred, int waitfor)
 #else				/* AFS_OSF_ENV */
 #if defined(AFS_SGI_ENV) || defined(AFS_SUN53_ENV)
-afs_fsync(OSI_VC_ARG(avc), flag, acred
+afs_fsync(OSI_VC_DECL(avc), int flag, struct AFS_UCRED *acred
 #ifdef AFS_SGI65_ENV
-	  , start, stop
-#endif
+	  , off_t start, off_t stop
+#endif /* AFS_SGI65_ENV */
     )
-#else
-afs_fsync(avc, acred)
-#endif
-#endif
-OSI_VC_DECL(avc);
-     struct AFS_UCRED *acred;
-#if defined(AFS_SGI_ENV) || defined(AFS_SUN53_ENV)
-     int flag;
-#ifdef AFS_SGI65_ENV
-     off_t start, stop;
-#endif
+#else /* !OSF && !SUN53 && !SGI */
+afs_fsync(OSI_VC_DECL(avc), struct AFS_UCRED *acred)
+#endif 
 #endif
 {
     register afs_int32 code;
     struct vrequest treq;
-    OSI_VC_CONVERT(avc)
+    OSI_VC_CONVERT(avc);
 
-	if (avc->vc_error)
+    if (avc->vc_error)
 	return avc->vc_error;
 
 #if defined(AFS_SUN5_ENV)
