@@ -11,7 +11,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_nfsclnt.c,v 1.13.6.3 2006/07/31 21:27:38 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_nfsclnt.c,v 1.13.6.7 2007/10/16 21:56:45 shadow Exp $");
 
 #if !defined(AFS_NONFSTRANS) || defined(AFS_AIX_IAUTH_ENV)
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -198,7 +198,17 @@ afs_nfsclient_reqhandler(struct afs_exporter *exporter,
     }
 /*    ObtainWriteLock(&afs_xnfsreq); */
     pag = PagInCred(*cred);
+#if defined(AFS_SUN510_ENV)
+    uid = crgetuid(*cred);
+#else
     uid = (*cred)->cr_uid;
+#endif
+    /* Do this early, so pag management knows */
+#ifdef	AFS_OSF_ENV
+    (*cred)->cr_ruid = NFSXLATOR_CRED;	/* Identify it as nfs xlator call */
+#else
+    (*cred)->cr_rgid = NFSXLATOR_CRED;	/* Identify it as nfs xlator call */
+#endif
     if ((afs_nfsexporter->exp_states & EXP_CLIPAGS) && pag != NOPAG) {
 	uid = pag;
     } else if (pag != NOPAG) {
@@ -296,11 +306,6 @@ afs_nfsclient_reqhandler(struct afs_exporter *exporter,
     *pagparam = pag;
     *outexporter = (struct afs_exporter *)np;
     afs_PutUser(au, WRITE_LOCK);
-#ifdef	AFS_OSF_ENV
-    (*cred)->cr_ruid = NFSXLATOR_CRED;	/* Identify it as nfs xlator call */
-#else
-    (*cred)->cr_rgid = NFSXLATOR_CRED;	/* Identify it as nfs xlator call */
-#endif
 /*    ReleaseWriteLock(&afs_xnfsreq);	*/
     return 0;
 }
@@ -479,11 +484,10 @@ afs_nfsclient_sysname(register struct nfsclientpag *np, char *inname,
 	MReleaseWriteLock(&afs_xnfspag);
     }
     if (inname) {
-	if (np->sysname) {
 	    for(count=0; count < np->sysnamecount;++count) {
 		afs_osi_Free(np->sysname[count], MAXSYSNAME);
+		np->sysname[count] = NULL;
 	    }
-	}
 	for(count=0; count < *num;++count) {
 	    np->sysname[count]= afs_osi_Alloc(MAXSYSNAME);
 	}
@@ -494,13 +498,13 @@ afs_nfsclient_sysname(register struct nfsclientpag *np, char *inname,
 	    cp += t+1;
 	}
 	np->sysnamecount = *num;
-    } else if (!np->sysnamecount) {
-	return ENODEV;      /* XXX */
     }
     if (allpags >= 0) {
 	/* Don't touch our arguments when called recursively */
 	*outname = np->sysname;
 	*num = np->sysnamecount;
+	if (!np->sysname[0])
+	    return ENODEV; /* XXX */
     }
     return 0;
 }
@@ -514,6 +518,7 @@ afs_nfsclient_GC(exporter, pag)
 {
     register struct nfsclientpag *np, **tnp, *nnp;
     register afs_int32 i, delflag;
+	int count;
 
 #if defined(AFS_SGIMP_ENV)
     osi_Assert(ISAFS_GLOCK());
@@ -529,8 +534,9 @@ afs_nfsclient_GC(exporter, pag)
 	    if ((pag == -1) || (!pag && delflag)
 		|| (pag && (np->refCount == 0) && (np->pag == pag))) {
 		*tnp = np->next;
-		if (np->sysname)
-		    afs_osi_Free(np->sysname, MAXSYSNAME);
+		for(count=0; count < np->sysnamecount;++count) {
+			afs_osi_Free(np->sysname[count], MAXSYSNAME);
+		}
 		afs_osi_Free(np, sizeof(struct nfsclientpag));
 	    } else {
 		tnp = &np->next;
