@@ -12,6 +12,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/budb/ol_verify.c,v 1.13.14.1 2007/07/06 11:40:40 jaltman Exp $");
 
 #include <stdio.h>
 #ifdef AFS_NT40_ENV
@@ -20,7 +22,13 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #include <afs/stds.h>
 #include <sys/types.h>
 #include <lock.h>
@@ -1493,8 +1501,8 @@ DbVerify(call, status, orphans, host)
  *	do a simple sanity check on the database header
  */
 
-void
-check_header(char *callerst)
+check_header(callerst)
+     char *callerst;
 {
     static int iteration_count = 0;
     afs_int32 eof;
