@@ -10,6 +10,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/dir/dir.c,v 1.24.4.3 2007/10/11 17:46:37 shadow Exp $");
 
 #ifdef KERNEL
 #if !defined(UKERNEL)
@@ -68,6 +70,7 @@ extern void *DNew();
 
 #include "afs/dir.h"
 
+#include "afs/longc_procs.h"
 #ifdef AFS_LINUX20_ENV
 #include "h/string.h"
 #endif
@@ -97,7 +100,13 @@ extern void *DNew();
 #include <netinet/in.h>
 #endif
 
+#ifdef HAVE_STRING_H
 #include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #endif /* KERNEL */
 
 afs_int32 DErrno;
