@@ -10,6 +10,8 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
+RCSID
+    ("$Header: /cvs/openafs/src/rx/SOLARIS/rx_knet.c,v 1.20.6.1 2007/10/05 02:24:39 shadow Exp $");
 
 #ifdef AFS_SUN5_ENV
 #include "rx/rx_kcommon.h"
@@ -48,17 +50,10 @@
 /*
  * Function pointers for kernel socket routines
  */
-#ifdef SOLOOKUP_TAKES_SOCKPARAMS
-struct sonode *(*sockfs_socreate)
-  (struct sockparams *, int, int, int, int, int *) = NULL;
-int (*sockfs_solookup)
-  (int, int, int, struct sockparams **) = NULL;
-#else
 struct sonode *(*sockfs_socreate)
   (vnode_t *, int, int, int, int, struct sonode *, int *) = NULL;
 struct vnode *(*sockfs_solookup)
   (int, int, int, char *, int *) = NULL;
-#endif /* SOLOOKUP_TAKES_SOCKPARAMS */
 int (*sockfs_sobind)
   (struct sonode *, struct sockaddr *, int, int, int) = NULL;
 int (*sockfs_sorecvmsg)
@@ -74,10 +69,6 @@ void (*sockfs_sockfree)
   (struct sonode *);
 #endif
 
-#ifndef UDP_MOD_NAME
-#define UDP_MOD_NAME "udp"
-#endif
-
 static afs_uint32 myNetAddrs[ADDRSPERSITE];
 static int myNetMTUs[ADDRSPERSITE];
 static int numMyNetAddrs = 0;
@@ -332,19 +323,12 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     struct sockaddr_in addr;
     int error;
     int len;
-#ifdef SOLOOKUP_TAKES_SOCKPARAMS
-    struct sockparams *sp;
-#endif
 
     AFS_STATCNT(osi_NewSocket);
 
     if (sockfs_solookup == NULL) {
 	sockfs_solookup =
-#ifdef SOLOOKUP_TAKES_SOCKPARAMS
-	    (int (*)())modlookup("sockfs", "solookup");
-#else
 	    (struct vnode * (*)())modlookup("sockfs", "solookup");
-#endif
 	if (sockfs_solookup == NULL) {
 	    return NULL;
 	}
@@ -393,14 +377,6 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     }
 #endif
 
-#ifdef SOLOOKUP_TAKES_SOCKPARAMS
-    error = sockfs_solookup(AF_INET, SOCK_DGRAM, 0, &sp);
-    if (error != 0) {
-	return NULL;
-    }
-
-    so = sockfs_socreate(sp, AF_INET, SOCK_DGRAM, 0, SOV_STREAM, &error);
-#else
     accessvp = sockfs_solookup(AF_INET, SOCK_DGRAM, 0, "/dev/udp", &error);
     if (accessvp == NULL) {
 	return NULL;
@@ -408,8 +384,6 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
 
     so = sockfs_socreate(accessvp, AF_INET, SOCK_DGRAM, 0, SOV_STREAM, NULL,
 			 &error);
-#endif /* SOLOOKUP_TAKES_SOCKPARAMS */
-
     if (so == NULL) {
 	return NULL;
     }
@@ -445,7 +419,7 @@ rxk_NewSocket(short aport)
 }
 
 int
-osi_FreeSocket(register osi_socket asocket)
+osi_FreeSocket(register osi_socket *asocket)
 {
     extern int rxk_ListenerPid;
     struct sonode *so = (struct sonode *)asocket;
@@ -470,11 +444,7 @@ osi_FreeSocket(register osi_socket asocket)
 
     /* Was sockfs_sounbind(so, 0); sockfs_sockfree(so); That's wrong */
     vp = SOTOV(so);
- #ifdef AFS_SUN511_ENV
-    VOP_CLOSE(vp, FREAD|FWRITE, 1, (offset_t)0, CRED(), NULL);
- #else
     VOP_CLOSE(vp, FREAD|FWRITE, 1, (offset_t)0, CRED());
- #endif
     VN_RELE(vp);
 
     return 0;
@@ -591,8 +561,6 @@ osi_NetReceive(osi_socket so, struct sockaddr_in *addr, struct iovec *dvec,
 /* set afs:afs_if_poll_interval = integer (value is in seconds)         */
 static int afs_if_poll_interval = 30;
 
-static timeout_id_t afs_if_poller_timeout = 0;
-
 /* Global array which holds the interface info for consumers            */
 struct afs_ifinfo afsifinfo[ADDRSPERSITE];
 
@@ -607,8 +575,8 @@ void
 osi_NetIfPoller()
 {
     cred_t *cr;
-    ldi_ident_t li = NULL;
-    ldi_handle_t lh = NULL;
+    ldi_ident_t li;
+    ldi_handle_t lh;
     struct lifnum lifn;
     struct lifconf lifc;
     struct lifreq lifr;
@@ -616,7 +584,7 @@ osi_NetIfPoller()
     struct sockaddr_in *sin4_local;
     struct sockaddr_in *sin4_dst;
     major_t udpmajor;
-    caddr_t lifcbuf = NULL;
+    caddr_t lifcbuf;
     int i, count, error, rv;
     int ifcount;
     int metric;
@@ -631,18 +599,15 @@ osi_NetIfPoller()
     udpmajor = ddi_name_to_major(UDP_MOD_NAME);
 
     error = ldi_ident_from_major(udpmajor, &li);
-    if (error) {
-        cmn_err(CE_WARN, "osi_NetIfPoller: ldi_ident_from_major failed: %d",
+    if (error)
+        cmn_err(CE_PANIC, "osi_NetIfPoller: ldi_ident_from_major failed: %d",
             error);
-        goto cleanup;
-    }
 
     error = ldi_open_by_name(UDP_DEV_NAME, FREAD, cr, &lh, li);
-    if (error) {
-        cmn_err(CE_WARN,
+    if (error)
+        cmn_err(CE_PANIC,
             "osi_NetIfPoller: ldi_open_by_name failed: %d", error);
-        goto cleanup;
-    }
+
 
     /* First, how many interfaces do we have? */
     (void) bzero((void *)&lifn, sizeof(struct lifnum));
@@ -650,11 +615,10 @@ osi_NetIfPoller()
 
     error = ldi_ioctl(lh, SIOCGLIFNUM, (intptr_t)&lifn,
         FKIOCTL, cr, &rv);
-    if (error) {
-        cmn_err(CE_WARN,
-                "osi_NetIfPoller: ldi_ioctl: SIOCGLIFNUM failed: %d", error);
-        goto cleanup;
-    }
+    if (error)
+     cmn_err(CE_PANIC,
+         "osi_NetIfPoller: ldi_ioctl: SIOCGLIFNUM failed: %d", error);
+
     ifcount = lifn.lifn_count;
 
     /* Set up some stuff for storing the results of SIOCGLIFCONF */
@@ -670,11 +634,10 @@ osi_NetIfPoller()
     /* Get info on each of our available interfaces. */
     error = ldi_ioctl(lh, SIOCGLIFCONF, (intptr_t)&lifc,
         FKIOCTL, cr, &rv);
-    if (error) {
-        cmn_err(CE_WARN,
-                "osi_NetIfPoller: ldi_ioctl: SIOCGLIFCONF failed: %d", error);
-        goto cleanup;
-    }
+    if (error)
+        cmn_err(CE_PANIC,
+            "osi_NetIfPoller: ldi_ioctl: SIOCGLIFCONF failed: %d", error);
+
     lifrp = lifc.lifc_req;
 
     count = 0;
@@ -694,12 +657,10 @@ osi_NetIfPoller()
         /* Get this interface's Flags */
         error = ldi_ioctl(lh, SIOCGLIFFLAGS, (intptr_t)&lifr,
             FKIOCTL, cr, &rv);
-        if (error) {
-            cmn_err(CE_WARN,
-                    "osi_NetIfPoller: ldi_ioctl: SIOCGLIFFLAGS failed: %d",
+        if (error)
+            cmn_err(CE_PANIC,
+                "osi_NetIfPoller: ldi_ioctl: SIOCGLIFFLAGS failed: %d",
                     error);
-            goto cleanup;
-        }
 
         /* Ignore plumbed but down interfaces. */
         if ((lifr.lifr_flags & IFF_UP) == 0)
@@ -750,44 +711,16 @@ osi_NetIfPoller()
 
     } /* Bottom of loop: for each interface ... */
 
-  cleanup:
-    /* End of thread. Time to clean up */
-    if (lifcbuf)
-        kmem_free(lifcbuf, ifcount * sizeof(struct lifreq));
-    if (lh)
-        (void) ldi_close(lh, FREAD, cr);
-    if (li)
-        (void) ldi_ident_release(li);
+    kmem_free(lifcbuf, ifcount * sizeof(struct lifreq));
 
-    if (afs_shuttingdown) {
-	/* do not schedule to run again if we're shutting down */
-	return;
-    }
+    /* End of thread. Time to clean up */
+    (void) ldi_close(lh, FREAD, cr);
+    (void) ldi_ident_release(li);
 
     /* Schedule this to run again after afs_if_poll_interval seconds */
-    afs_if_poller_timeout = timeout((void(*) (void *)) osi_StartNetIfPoller,
-        NULL, drv_usectohz((clock_t)afs_if_poll_interval * MICROSEC));
-}
+    (void) timeout((void(*) (void *)) osi_StartNetIfPoller, NULL,
+        drv_usectohz((clock_t)afs_if_poll_interval * MICROSEC));
 
-void
-osi_StopNetIfPoller(void)
-{
-    /* it's okay if untimeout races with StartNetIfPoller/NetIfPoller;
-     * it can handle being passed invalid ids. If StartNetIfPoller is
-     * in the middle of running, untimeout will not return until
-     * StartNetIfPoller is done */
-    untimeout(afs_if_poller_timeout);
-
-    /* if NetIfPoller is queued or running, ddi_taskq_destroy will not
-     * return until it is done */
-    ddi_taskq_destroy(afs_taskq);
-
-    rw_destroy(&afsifinfo_lock);
-
-    if (afs_termState == AFSOP_STOP_NETIF) {
-	afs_termState = AFSOP_STOP_COMPLETE;
-	osi_rxWakeup(&afs_termState);
-    }
 }
 #endif /* AFS_SUN510_ENV */
 
