@@ -18,7 +18,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/vol/vutil.c,v 1.14 2003/11/15 04:59:14 shadow Exp $");
+    ("$Header: /cvs/openafs/src/vol/vutil.c,v 1.16 2004/08/19 00:22:38 kolya Exp $");
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -109,8 +109,10 @@ VCreateVolume(Error * ec, char *partname, VolId volumeId, VolId parentId)
 {				/* Should be the same as volumeId if there is
 				 * no parent */
     Volume *retVal;
-    VOL_LOCK retVal = VCreateVolume_r(ec, partname, volumeId, parentId);
-    VOL_UNLOCK return retVal;
+    VOL_LOCK;
+    retVal = VCreateVolume_r(ec, partname, volumeId, parentId);
+    VOL_UNLOCK;
+    return retVal;
 }
 
 Volume *
@@ -252,21 +254,21 @@ VCreateVolume_r(Error * ec, char *partname, VolId volumeId, VolId parentId)
     IH_INIT(handle, device, vol.parentId, tempHeader.volumeInfo);
     fdP = IH_OPEN(handle);
     if (fdP == NULL) {
-	Log("VCreateVolume:  Problem iopen inode %llu (err=%d)\n",
-	    (afs_uintmax_t) tempHeader.volumeInfo, errno);
+	Log("VCreateVolume:  Problem iopen inode %s (err=%d)\n",
+	    PrintInode(NULL, tempHeader.volumeInfo), errno);
 	unlink(volumePath);
 	goto bad;
     }
     if (FDH_SEEK(fdP, 0, SEEK_SET) < 0) {
-	Log("VCreateVolume:  Problem lseek inode %llu (err=%d)\n",
-	    (afs_uintmax_t) tempHeader.volumeInfo, errno);
+	Log("VCreateVolume:  Problem lseek inode %s (err=%d)\n",
+	    PrintInode(NULL, tempHeader.volumeInfo), errno);
 	FDH_REALLYCLOSE(fdP);
 	unlink(volumePath);
 	goto bad;
     }
     if (FDH_WRITE(fdP, (char *)&vol, sizeof(vol)) != sizeof(vol)) {
-	Log("VCreateVolume:  Problem writing to  inode %llu (err=%d)\n",
-	    (afs_uintmax_t) tempHeader.volumeInfo, errno);
+	Log("VCreateVolume:  Problem writing to  inode %s (err=%d)\n",
+	    PrintInode(NULL, tempHeader.volumeInfo), errno);
 	FDH_REALLYCLOSE(fdP);
 	unlink(volumePath);
 	goto bad;
@@ -289,8 +291,10 @@ VCreateVolume_r(Error * ec, char *partname, VolId volumeId, VolId parentId)
 void
 AssignVolumeName(register VolumeDiskData * vol, char *name, char *ext)
 {
-    VOL_LOCK AssignVolumeName_r(vol, name, ext);
-VOL_UNLOCK}
+    VOL_LOCK;
+    AssignVolumeName_r(vol, name, ext);
+    VOL_UNLOCK;
+}
 
 void
 AssignVolumeName_r(register VolumeDiskData * vol, char *name, char *ext)
@@ -335,15 +339,19 @@ CopyVolumeHeader(VolumeDiskData * from, VolumeDiskData * to)
 {
     afs_int32 code;
 
-    VOL_LOCK code = CopyVolumeHeader_r(from, to);
-    VOL_UNLOCK return (code);
+    VOL_LOCK;
+    code = CopyVolumeHeader_r(from, to);
+    VOL_UNLOCK;
+    return (code);
 }
 
 void
 ClearVolumeStats(register VolumeDiskData * vol)
 {
-    VOL_LOCK ClearVolumeStats_r(vol);
-VOL_UNLOCK}
+    VOL_LOCK;
+    ClearVolumeStats_r(vol);
+    VOL_UNLOCK;
+}
 
 void
 ClearVolumeStats_r(register VolumeDiskData * vol)
